// List of view-names that need type differentiation
var overlap = ["userId","collectedUsersOwners","startTime","endTime"]
// Strings prepended to view-name for localization
var view_alias = {"user":"user", "account":"account", "permission":"permission", "permission_to_holders":"holder", "permissions_from_holder":"holder", "permission_hierarchy_to_child":"hierarchy", "permission_hierarchy_to_parent":"hierarchy"}


/*
Return the name of the collector
*/
var getCollectorName = function (JsonStr) {
	return JSON.parse(JsonStr)["collector-name"];
}


/*
Get changes to the service parameters between data source configurations
Returns CTE string containing changes to service parameters
*/
var getServiceParamChanges = function (beforeJsonStr, afterJsonStr, dbPlatform) {

	var beforeJSON = JSON.parse(beforeJsonStr);
	var afterJSON = JSON.parse(afterJsonStr);

	var beforeMap = []; // Associative array to store parameter, value pairs for before configuration 
	var afterMap = []; // Associative array to store parameter, value pairs for after configuration 

	// Add parameter, value pairs to beforeMap
	for (i = 0; i < beforeJSON["service-parms"].length; i++) {
		// If service parameter has options get value from options
		if(beforeJSON["service-parms"][i]["options"] != null && !beforeJSON["service-parms"][i]["hidden"]){
			for(j = 0; j < beforeJSON["service-parms"][i]["options"].length; j++){
				if(beforeJSON["service-parms"][i]["options"][j]["value"] == beforeJSON["service-parms"][i]["value"]){
					beforeMap[beforeJSON["service-parms"][i]["display-name"]] = beforeJSON["service-parms"][i]["options"][j]["display-name"];
					break;
				}
			}
			// if option not found add the value with the name added to the front
			if(!(beforeJSON["service-parms"][i]["display-name"] in beforeMap)){
				beforeMap[beforeJSON["service-parms"][i]["display-name"]] = beforeJSON["service-parms"][i]["name"] + "_" + beforeJSON["service-parms"][i]["value"];
			}
		}
		// If there are no options get value
		else{
			// Only add non-hidden parameters
			if(!beforeJSON["service-parms"][i]["hidden"]){
				beforeMap[beforeJSON["service-parms"][i]["display-name"]] = beforeJSON["service-parms"][i]["value"];
			}
		}
	}

	// Add parameter, value pairs to afterMap
	for (i = 0; i < afterJSON["service-parms"].length; i++) {
		// If service parameter has options get value from options
		if(afterJSON["service-parms"][i]["options"] != null && !afterJSON["service-parms"][i]["hidden"]){
			for(j = 0; j < afterJSON["service-parms"][i]["options"].length; j++){
				if(afterJSON["service-parms"][i]["options"][j]["value"] == afterJSON["service-parms"][i]["value"]){
					afterMap[afterJSON["service-parms"][i]["display-name"]] = afterJSON["service-parms"][i]["options"][j]["display-name"];
					break;
				}
			}
			// if option not found add the value with the name added to the front
			if(!(afterJSON["service-parms"][i]["display-name"] in afterMap)){
				afterMap[beforeJSON["service-parms"][i]["display-name"]] = afterJSON["service-parms"][i]["name"] + "_" + afterJSON["service-parms"][i]["value"];
			}
		}
		
		else{ // If there are no options
			if(!afterJSON["service-parms"][i]["hidden"]){ // Only add non-hidden parameters
				afterMap[afterJSON["service-parms"][i]["display-name"]] = afterJSON["service-parms"][i]["value"];
			}	
		}
	}

	// Get differences between the arrays
	var mapDiff = getMapDiff(beforeMap, afterMap, "service-parms");

	return createCTE(mapDiff,"service-parms", "", 0, 0, dbPlatform);
}


/*
Get differences between scrips between data source configurations
Returns CTE string containing changed transformation scripts
*/
var getScriptChanges = function(beforeJsonStr, afterJsonStr, dbPlatform){
	
	var beforeJSON = JSON.parse(beforeJsonStr);
	var afterJSON = JSON.parse(afterJsonStr);

	var beforeMap = []; // Associative array to store name, script pairs for before configuration
	var afterMap = []; // Associative array to store name, script pairs for after configuration

	// Add values to the arrays
	for (i = 0; i < beforeJSON["ecma-scripts"].length; i++) {
		beforeMap[beforeJSON["ecma-scripts"][i]["name"]] = beforeJSON["ecma-scripts"][i]["script"]
	}

	for (i = 0; i < afterJSON["ecma-scripts"].length; i++) {
		afterMap[afterJSON["ecma-scripts"][i]["name"]] = afterJSON["ecma-scripts"][i]["script"]
	}

	// Get differences between the arrays
	var mapDiff = getMapDiff(beforeMap, afterMap, "ecma-scripts");
	
	// Return a CTE with the differences
	return createCTE(mapDiff,"ecma-scripts", "", 1, 0, dbPlatform);
}


/*
This method gets a list of view parameters that have changes. In the UI these are the fields above the collection attributes (e.g Column Delimiter)
These come from  views -> view-parms
Returns CTE string containing parameter changes for all views
*/
var getChangedViewParams = function (beforeJsonStr, afterJsonStr, dbPlatform) {

	var beforeJSON = JSON.parse(beforeJsonStr);
	var afterJSON = JSON.parse(afterJsonStr);

	var changedViewPerms = []

	var cte = "";

	// Loop through the views in the configuration
	for (i = 0; i < beforeJSON["views"].length; i++) {

		var beforeMap = []; // Associative array to store parameter, value pairs for before configuration 
		var afterMap = []; // Associative array to store parameter, value pairs for after configuration 

		// Add parameter, value pairs to beforeMap
		for (j = 0; j < beforeJSON["views"][i]["view-parms"].length; j++) {
			// If there are options, get the value from the options
			if('options' in beforeJSON["views"][i]["view-parms"][j] && beforeJSON["views"][i]["view-parms"][j]["options"] != null){
				for(k = 0; k < beforeJSON["views"][i]["view-parms"][j]["options"].length; k++){
					if(beforeJSON["views"][i]["view-parms"][j]["value"] === beforeJSON["views"][i]["view-parms"][j]["options"][k]["value"]){
						beforeMap[beforeJSON["views"][i]["view-parms"][j]["display-name"]] = beforeJSON["views"][i]["view-parms"][j]["options"][k]["display-name"];
					}
				}
				// If the options is not found add the value with the parameter name added to the front
				if(!(beforeJSON["views"][i]["view-parms"][j]["display-name"] in beforeMap)){
					beforeMap[beforeJSON["views"][i]["view-parms"][j]["display-name"]] = beforeJSON["views"][i]["view-parms"][j]["name"] + "_" + beforeJSON["views"][i]["view-parms"][j]["value"];
				}
			}
			// If there are no options, get the value
			else{
				beforeMap[beforeJSON["views"][i]["view-parms"][j]["display-name"]] = beforeJSON["views"][i]["view-parms"][j]["value"];
			}
		}

		// Add parameter, value pairs to afterMap
		for (j = 0; j < afterJSON["views"][i]["view-parms"].length; j++) {
			// If there are options, get the value from the options
			if('options' in afterJSON["views"][i]["view-parms"][j] && afterJSON["views"][i]["view-parms"][j]["options"] != null){
				for(k = 0; k < afterJSON["views"][i]["view-parms"][j]["options"].length;k++){
					if(afterJSON["views"][i]["view-parms"][j]["value"] === afterJSON["views"][i]["view-parms"][j]["options"][k]["value"]){
						afterMap[afterJSON["views"][i]["view-parms"][j]["display-name"]] = afterJSON["views"][i]["view-parms"][j]["options"][k]["display-name"];
					}
				}
				// If the options is not found add the value with the parameter name added to the front
				if(!(afterJSON["views"][i]["view-parms"][j]["display-name"] in afterMap)){
					afterMap[beforeJSON["views"][i]["view-parms"][j]["display-name"]] = afterJSON["views"][i]["view-parms"][j]["name"] + "_" + afterJSON["views"][i]["view-parms"][j]["value"];
				}
			}
			// If there are no options, get the value
			else{
				afterMap[afterJSON["views"][i]["view-parms"][j]["display-name"]] = afterJSON["views"][i]["view-parms"][j]["value"];
			}
		}

		// Get differences between the arrays
		var mapDiff = getMapDiff(beforeMap, afterMap, "view-parms");
		
		// Add UNION ALL if the CTE is not empty
		if(cte != ""){
			cte += "UNION ALL\n"
		}

		// Add Section for view in CTE
		cte += createCTE(mapDiff, "view-parms", beforeJSON["views"][i]["name"], 2, i, dbPlatform);

	}

	// Return CTE containing parameter changes for all views
	return cte;
}


/*
This method gets a list of attributes that have changed for each sub collector(view) (e.g. permission to holder mapping).
These come from  views -> schema-map-filter -> generic-map
Returns CTE string containing generic-map item changes for all views
*/
var getChangedGenMaps = function (beforeJsonStr, afterJsonStr, localizer, dbPlatform) {

	var beforeJSON = JSON.parse(beforeJsonStr);
	var afterJSON = JSON.parse(afterJsonStr);

	var cte = "";

	// Loop through each item in the views list
	for (i = 0; i < beforeJSON["views"].length; i++) {

		var beforeMap = []; // Associative array to store attribute, value pairs for before configuration 
		var afterMap = []; // Associative array to store attribute, value pairs for after configuration 

		// Add attribute, value pairs to beforeMap
		for (j = 0; j < beforeJSON["views"][i]["schema-map-filter"]["generic-map"].length; j++) {
			if(!beforeJSON["views"][i]["schema-map-filter"]["generic-map"][j]["hidden"] && !beforeJSON["views"][i]["schema-map-filter"]["generic-map"][j]["read-only-mapping"]){
				beforeMap[beforeJSON["views"][i]["schema-map-filter"]["generic-map"][j]["view-name"]] = beforeJSON["views"][i]["schema-map-filter"]["generic-map"][j]["app-name"]
			}
		}

		// Add attribute, value pairs to afterMap
		for (j = 0; j < afterJSON["views"][i]["schema-map-filter"]["generic-map"].length; j++) {
			if(!afterJSON["views"][i]["schema-map-filter"]["generic-map"][j]["hidden"] && !afterJSON["views"][i]["schema-map-filter"]["generic-map"][j]["read-only-mapping"]){
				afterMap[afterJSON["views"][i]["schema-map-filter"]["generic-map"][j]["view-name"]] = afterJSON["views"][i]["schema-map-filter"]["generic-map"][j]["app-name"]
			}
		}

		// Get differences between the arrays
		var mapDiff = getMapDiff(beforeMap, afterMap, view_alias[beforeJSON["views"][i]["name"]], localizer);

		// Add UNION ALL if the CTE is not empty
		if(cte != ""){
			cte += "UNION ALL\n"
		}

		// Add Section for view in CTE
		cte += createCTE(mapDiff, "generic-map", beforeJSON["views"][i]["name"], 3, i, dbPlatform);
	}

	// Return CTE containing attribute changes for all views
	return cte;
}


/*
Function to get the difference between two associative arrays
*/
var getMapDiff = function (beforeMap, afterMap, type, localizer) {

	var paramNames = []; // array to store parameter names
	var beforeValues = []; // array to store before values
	var afterValues = []; // array to store after values

	for (var key in beforeMap) {
		if (key in afterMap) {
			// If key is in both beforeMap and afterMap
			if (beforeMap[key] != afterMap[key]) {
				// If values are different add to the arrays
				if (localizer != undefined) {
					if(overlap.indexOf(key)!=-1){
						paramNames.push(localizer.getString((type + key).toUpperCase()));
					} else{
						paramNames.push(localizer.getString((key).toUpperCase()));
					}
				}
				else {
					paramNames.push(key);
				}
				if(type === "ecma-scripts"){
					// This is to prevent long scripts from causing errors in oracle. Only need to know if the scrips are different.
					beforeValues.push("script_a");
					afterValues.push("script_b");
				} else{
					beforeValues.push(beforeMap[key]);
					afterValues.push(afterMap[key]);
				}
			}
			// Remove key from beforeMap and afterMap
			delete beforeMap[key];
			delete afterMap[key];
		} else {
			// Keys only in beforeMap
			if (beforeMap[key] != "") {
				if (localizer != undefined) {
					if(overlap.indexOf(key)!=-1){
						paramNames.push(localizer.getString((type + key).toUpperCase()));
					} else{
						paramNames.push(localizer.getString((key).toUpperCase()));
					}
				}
				else {
					paramNames.push(key);
				}
				if(type === "ecma-scripts"){
					// This is to prevent long scripts from causing errors in oracle.
					beforeValues.push("script_a");
				} else{
					beforeValues.push(beforeMap[key]);
				}
				afterValues.push("");
				delete beforeMap[key];
			}
		}
	}
	// Loop through remaining keys in afterMap
	for (var key in afterMap) {
		// Add parameters that are only in afterMap to arrays
		if (afterMap[key] != "") {
			if (localizer != undefined) {
				if(overlap.indexOf(key)!=-1){
					paramNames.push(localizer.getString((type + key).toUpperCase()));
				} else{
					paramNames.push(localizer.getString((key).toUpperCase()));
				}
			}
			else {
				paramNames.push(key);
			}
			beforeValues.push("");
			if(type === "ecma-scripts"){
				// This is to prevent long scripts from causing errors in oracle.
				afterValues.push("script_b");
			} else {
				afterValues.push(afterMap[key]);
			}
			delete afterMap[key];
		}
	}

	// Return array containing the arrays for names, before values, and after values 
	return [paramNames, beforeValues, afterValues];
}


/*
Function to creates a CTE that jaspersoft can use as the query in a subreport.
The cte contains the columns type, view_name, attr_name, before_value, and after_value.
*/
var createCTE = function(mapDiff, attrType, viewName, typeOrder, viewOrder, dbPlatform){
	var cte = '';

	for(var i = 0; i < mapDiff[0].length; i++){
		if(i > 0){
			// Add 'UNION ALL' if not the first row 
			cte += "UNION ALL\n";
		}
		// Column values for the row
		cte += ("--NOT_EMPTY\n"); // Used for jaspersoft print when statement
		cte += ("SELECT\n");
		cte += ("'" + attrType + "' AS type,\n");
		cte += ("'" + viewName + "' AS view_name,\n");
		cte += ("'" + mapDiff[0][i] + "' AS attr_name,\n");
		cte += ("'" + mapDiff[1][i] + "' AS before_value,\n");
		cte += ("'" + mapDiff[2][i] + "' AS after_value,\n");
		cte += (i + " AS row_order,\n");
		cte += (viewOrder + " AS view_order,\n");
		cte += (typeOrder + " AS type_order\n");

		if(dbPlatform === "oracle"){
			// Oracle needs to select from 'DUAL'
			cte += "FROM DUAL\n";
		}
	}

	if(cte === ''){
		// Add row of null values if cte is empty.
		cte += ("SELECT\n");
		cte += ("NULL AS type,\n");
		cte += ("NULL AS view_name,\n");
		cte += ("NULL AS attr_name,\n");
		cte += ("NULL AS before_value,\n");
		cte += ("NULL AS after_value,\n");
		cte += ("0 AS row_order,\n");
		cte += ("0 AS view_order,\n");
		cte += ("0 AS type_order\n");

		if(dbPlatform === "oracle"){
			// Oracle needs to select from 'DUAL'
			cte += "FROM DUAL\n";
		}
	}

	return cte;
}